@@ -166,30 +166,35 @@ sub validateTypedRow
         if ( $changedFields->{primary_ns}->selectedType() eq 'eboxDNS' ) {
             my $dns = EBox::Global->modInstance('dns');
             unless ( $dns->isEnabled() ) {
-                throw EBox::Exceptions::External(__('DNS service must be active to be able to select eBox as primary DNS server'));
+                throw EBox::Exceptions::External(__('DNS module must be enabled to be able to select eBox as primary DNS server'));
+            }
+        }
+    }
+    # Validate NTP server
+    if ( exists $changedFields->{ntp_server} ) {
+        # Check if chosen is NTP to check if it is enabled
+        if ( $changedFields->{ntp_server}->selectedType() eq 'eboxNTP' ) {
+            my $ntp = EBox::Global->modInstance('ntp');
+            unless ( $ntp->isEnabled() ) {
+                throw EBox::Exceptions::External(__('NTP module must be enabled to be able to select eBox as NTP server'));
+            }
+        }
+    }
+    # Validate WINS server
+    if ( exists $changedFields->{wins_server} ) {
+        # Check if chosen is WINS to check if it is enabled
+        if ( $changedFields->{wins_server}->selectedType() eq 'eboxWINS' ) {
+            my $sambaMod = EBox::Global->modInstance('samba');
+            unless ( $sambaMod->isEnabled() ) {
+                throw EBox::Exceptions::External(
+                    __('Samba module must be enabled to be able to select eBox as WINS server')
+                   );
             }
         }
     }
 
 }
 
-# Method: formSubmitted
-#
-#       When the form is submitted, the model must set up the jabber
-#       dispatcher client service and sets the output rule in the
-#       firewall
-#
-# Overrides:
-#
-#      <EBox::Model::DataForm::formSubmitted>
-#
-sub formSubmitted
-  {
-
-      my ($self, $oldRow) = @_;
-
-  }
-
 # Method: defaultGateway
 #
 #     Get the current default gateway
@@ -280,6 +285,60 @@ sub nameserver
 
 }
 
+# Method: ntpServer
+#
+#     Get the current IP address from the NTP server
+#
+# Returns:
+#
+#     String - the current NTP server if any, otherwise undef is returned
+#
+sub ntpServer
+{
+    my ($self) = @_;
+
+    my $row = $self->row();
+
+    my $selectedType = $row->elementByName('ntp_server')->selectedType();
+
+    if ( $selectedType eq 'none' ) {
+        return undef;
+    } elsif ( $selectedType eq 'eboxNTP' ) {
+        my $ifaceAddr = $self->{netMod}->ifaceAddress($self->{interface});
+        return $ifaceAddr;
+    } elsif ( $selectedType eq 'custom_ntp' ) {
+        return $row->valueByName('custom_ntp');
+    }
+
+}
+
+# Method: winsServer
+#
+#     Get the current IP address from the WINS server
+#
+# Returns:
+#
+#     String - the current WINS server if any, otherwise undef is returned
+#
+sub winsServer
+{
+    my ($self) = @_;
+
+    my $row = $self->row();
+
+    my $selectedType = $row->elementByName('wins_server')->selectedType();
+
+    if ( $selectedType eq 'none' ) {
+        return undef;
+    } elsif ( $selectedType eq 'eboxWINS' ) {
+        my $ifaceAddr = $self->{netMod}->ifaceAddress($self->{interface});
+        return $ifaceAddr;
+    } elsif ( $selectedType eq 'custom_wins' ) {
+        return $row->valueByName('custom_wins');
+    }
+
+}
+
 # Method: headTitle
 #
 # Overrides:
@@ -291,8 +350,6 @@ sub headTitle
     return undef;
 }
 
-
-
 # Group: Protected methods
 
 # Method: _table
@@ -348,6 +405,34 @@ sub _table
                                         printableName => __('None'),
                                        ));
 
+    my @ntpSubtypes = ( new EBox::Types::Union::Text(fieldName     => 'none',
+                                                     printableName => __('None')));
+
+    if ( $gl->modExists('ntp') ) {
+        push(@ntpSubtypes,
+             new EBox::Types::Union::Text(fieldName     => 'eboxNTP',
+                                          printableName => __('local eBox NTP')));
+    }
+    push(@ntpSubtypes,
+         new EBox::Types::HostIP(fieldName     => 'custom_ntp',
+                                 printableName => __('Custom'),
+                                 editable      => 1)
+        );
+
+    my @winsSubtypes = ( new EBox::Types::Union::Text(fieldName    => 'none',
+                                                      printableName => __('None')));
+
+    if ( $gl->modExists('samba') ) {
+        push(@winsSubtypes,
+             new EBox::Types::Union::Text(fieldName     => 'eboxWINS',
+                                          printableName => __('local eBox')));
+    }
+    push(@winsSubtypes,
+         new EBox::Types::HostIP(fieldName     => 'custom_wins',
+                                 printableName => __('Custom'),
+                                 editable      => 1)
+        );
+
     my @tableDesc =
       (
        new EBox::Types::Union(
@@ -404,6 +489,22 @@ sub _table
                                editable      => 1,
                                optional      => 1,
                               ),
+       new EBox::Types::Union(
+                              fieldName      => 'ntp_server',
+                              printableName  => __('NTP server'),
+                              editable       => 1,
+                              subtypes       => \@ntpSubtypes,
+                              help           => __('If "eBox NTP" is present and selected, '
+                                                   . 'eBox will be the NTP server for DHCP clients'),
+                              ),
+       new EBox::Types::Union(
+                              fieldName      => 'wins_server',
+                              printableName  => __('WINS server'),
+                              editable       => 1,
+                              subtypes       => \@winsSubtypes,
+                              help           => __('If "eBox Samba" is present and selected, '
+                                                   . 'eBox will be the WINS server for DHCP clients'),
+                             ),
       );
 
       my $dataForm = {
